@@ -14,9 +14,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * @author: Olivier BERT
  */
@@ -26,8 +24,9 @@
 #endif
 
 #include <speechd_types.h>
+#include <safe_io.h>
 #include <errno.h>
-#include <sys/poll.h>
+#include <poll.h>
 #include <fcntl.h>
 #include <langinfo.h>
 #include <sys/stat.h>
@@ -41,6 +40,8 @@
 // #define DEBUG_MODULE 1
 DECLARE_DEBUG()
 
+static int initialized = 0;
+
 /* Thread and process control */
 static int cicero_speaking = 0;
 
@@ -146,6 +147,11 @@ int module_init(char **status_info)
 
 	DBG("Module init\n");
 
+	if (access(CiceroExecutable, X_OK) != 0) {
+		DBG("ERROR: can not reach executable %s", CiceroExecutable);
+		return -1;
+	}
+
 	(void)signal(SIGPIPE, SIG_IGN);
 
 	DBG("call the pipe system call\n");
@@ -158,19 +164,17 @@ int module_init(char **status_info)
 	stderr_redirect = open(CiceroExecutableLog,
 			       O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR);
 	if (stderr_redirect == -1) {
-		DBG("ERROR: Openning debug file for Cicero binary failed: (error=%d) %s", stderr_redirect, strerror(errno));
+		DBG("ERROR: Opening debug file for Cicero binary failed: (error=%d) %s", stderr_redirect, strerror(errno));
 	} else {
 		DBG("Cicero synthesizer logging to file %s",
 		    CiceroExecutableLog);
 	}
 	switch (fork()) {
-	case -1:
-		{
+	case -1:{
 			DBG("Error fork()\n");
 			return -1;
 		}
-	case 0:
-		{
+	case 0:{
 			if (dup2(fd2[0], 0) < 0	/* stdin */
 			    || dup2(fd1[1], 1) < 0) {	/* stdout */
 				DBG("Error dup2()\n");
@@ -191,8 +195,9 @@ int module_init(char **status_info)
 			DBG("Error execl()\n");
 			exit(1);
 		}
-	default:
-		{
+	default:{
+			if (stderr_redirect >= 0)
+				close(stderr_redirect);
 			close(fd1[1]);
 			close(fd2[0]);
 			if (fcntl(fd2[1], F_SETFL, O_NDELAY) < 0
@@ -206,7 +211,7 @@ int module_init(char **status_info)
 	cicero_message = g_malloc(sizeof(char *));
 	*cicero_message = NULL;
 
-	sem_init(&cicero_semaphore , 0, 0);
+	sem_init(&cicero_semaphore, 0, 0);
 
 	DBG("Cicero: creating new thread for cicero_tracking\n");
 	cicero_speaking = 0;
@@ -224,6 +229,8 @@ int module_init(char **status_info)
 
 	*status_info = g_strdup("Cicero initialized successfully.");
 
+	initialized = 1;
+
 	return 0;
 }
 
@@ -249,6 +256,7 @@ int module_speak(gchar * data, size_t bytes, SPDMessageType msgtype)
 		*cicero_message = NULL;
 	}
 	*cicero_message = module_strip_ssml(data);
+	/* TODO: use a generic engine for SPELL, CHAR, KEY */
 	cicero_message_type = SPD_MSGTYPE_TEXT;
 
 	/* Setting voice */
@@ -260,7 +268,7 @@ int module_speak(gchar * data, size_t bytes, SPDMessageType msgtype)
 	cicero_speaking = 1;
 	sem_post(&cicero_semaphore);
 
-	DBG("Cicero: leaving module_speak() normaly\n\r");
+	DBG("Cicero: leaving module_speak() normally\n\r");
 	return bytes;
 }
 
@@ -294,10 +302,15 @@ int module_close(void)
 		module_stop();
 	}
 
+	if (!initialized)
+		return 0;
+
 	if (module_terminate_thread(cicero_speaking_thread) != 0)
 		return -1;
 
-	sem_destroy(&cicero_semaphore );
+	sem_destroy(&cicero_semaphore);
+
+	initialized = 0;
 	return 0;
 }
 
@@ -377,9 +390,7 @@ void *_cicero_speak(void *nothing)
 						break;
 					}
 					if (ret > 0)
-						TEMP_FAILURE_RETRY(read
-								   (fd1[0], b,
-								    2));
+						safe_read(fd1[0], b, 2);
 					if (cicero_stop) {
 						cicero_speaking = 0;
 						module_report_event_stop();
