@@ -33,6 +33,7 @@
 #include "server.h"
 #include "sem_functions.h"
 #include "output.h"
+#include "fdsetconv.h"
 
 /*
   Parse() receives input data and parses them. It can
@@ -141,7 +142,7 @@ char *parse(const char *buf, const int bytes, const int fd)
 	} else {
 enddata:
 		/* In the end of the data flow we got a "\r\n." NEWLINE command. */
-		MSG(5, "Buffer: |%s| bytes:", buf, bytes);
+		MSG(5, "Buffer: |%s| %d bytes:", buf, bytes);
 
 		if (((bytes >= 5) && ((!strncmp(buf, "\r\n." NEWLINE, bytes))))
 		    || (end_data == 1)
@@ -280,24 +281,34 @@ char *parse_history(const char *buf, const int bytes, const int fd,
 		GET_PARAM_STR(hist_get_sub, 2, CONV_DOWN);
 
 		if (TEST_CMD(hist_get_sub, "client_list")) {
-			return (char *)history_get_client_list();
+			/* No longer able to get client list */
+			return g_strdup(ERR_NOT_IMPLEMENTED);
 		} else if (TEST_CMD(hist_get_sub, "client_id")) {
+			/* Can still get your client id */
 			return (char *)history_get_client_id(fd);
 		} else if (TEST_CMD(hist_get_sub, "client_messages")) {
 			int start, num;
 			char *who;
 			int who_id;
+			int client_id = get_client_uid_by_fd(fd);
 
 			/* TODO: This needs to be (sim || am)-plified */
 			who = get_param(buf, 3, bytes, 1);
 			CHECK_PARAM(who);
 			if (!strcmp(who, "self"))
+				/* TODO: Get all our messages, that should be allowed but how many to get... */
 				return g_strdup(ERR_NOT_IMPLEMENTED);
 			if (!strcmp(who, "all"))
+				/* No longer allowed, security hole here */
 				return g_strdup(ERR_NOT_IMPLEMENTED);
 			if (!isanum(who))
 				return g_strdup(ERR_NOT_A_NUMBER);
 			who_id = atoi(who);
+
+			/* Check if the id is the client */
+			if (who_id != client_id)
+				return g_strdup(ERR_NOT_IMPLEMENTED);
+
 			g_free(who);
 			GET_PARAM_INT(start, 4);
 			GET_PARAM_INT(num, 5);
@@ -582,6 +593,8 @@ char *parse_set(const char *buf, const int bytes, const int fd,
 
 		if (TEST_CMD(punct_s, "all"))
 			punctuation_mode = SPD_PUNCT_ALL;
+		else if (TEST_CMD(punct_s, "most"))
+			punctuation_mode = SPD_PUNCT_MOST;
 		else if (TEST_CMD(punct_s, "some"))
 			punctuation_mode = SPD_PUNCT_SOME;
 		else if (TEST_CMD(punct_s, "none"))
@@ -1039,6 +1052,11 @@ char *parse_get(const char *buf, const int bytes, const int fd,
 	} else if (TEST_CMD(get_type, "volume")) {
 		g_string_append_printf(result, C_OK_GET "-%d" NEWLINE OK_GET,
 				       settings->msg_settings.volume);
+	} else if (TEST_CMD(get_type, "punctuation")) {
+		char *punct = EPunctMode2str(settings->msg_settings.punctuation_mode);
+		g_string_append_printf(result, C_OK_GET "-%s" NEWLINE OK_GET,
+				       punct);
+		g_free(punct);
 	} else {
 		g_free(get_type);
 		g_string_append(result, ERR_PARAMETER_INVALID);
