@@ -15,9 +15,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * $Id: speaking.c,v 1.56 2008-10-15 18:06:48 hanke Exp $
  */
@@ -33,6 +31,7 @@
 #include "speechd.h"
 #include "server.h"
 #include "index_marking.h"
+#include "symbols.h"
 #include "module.h"
 #include "set.h"
 #include "alloc.h"
@@ -47,6 +46,16 @@ static SPDPriority highest_priority = 0;
 int SPEAKING = 0;
 int poll_count;
 
+OutputModule *speaking_module;
+int speaking_uid;
+int speaking_gid;
+
+/* Pause and resume handling */
+int pause_requested;
+int pause_requested_fd;
+int pause_requested_uid;
+int resume_requested;
+
 /*
   Speak() is responsible for getting right text from right
   queue in right time and saying it loud through the corresponding
@@ -60,6 +69,7 @@ void *speak(void *data)
 	struct pollfd main_pfd;
 	struct pollfd helper_pfd;
 	int revents;
+	OutputModule *output;
 
 	/* Block all signals and set thread states */
 	set_speak_thread_attributes();
@@ -218,6 +228,44 @@ void *speak(void *data)
 			continue;
 		}
 
+		/* Choose the output module */
+		output = get_output_module(message);
+		if (output == NULL) {
+			MSG(3, "Output module doesn't work...");
+			output_check_module(output);
+			pthread_mutex_unlock(&element_free_mutex);
+			continue;
+		}
+
+		int punct_missing = 0;
+		if (strcmp(output->name, "flite") == 0 ||
+		    strcmp(output->name, "dtk-generic") == 0 ||
+		    strcmp(output->name, "epos-generic") == 0 ||
+		    strcmp(output->name, "llia_phon-generic") == 0 ||
+		    strcmp(output->name, "mary-generic") == 0 ||
+		    strcmp(output->name, "pico-generic") == 0 ||
+		    strcmp(output->name, "swift-generic") == 0 ||
+		    strcmp(output->name, "pico") == 0)
+			/* These don't support punctuation */
+			/* FIXME: rather make them express it */
+			punct_missing = 1;
+
+		if (message->settings.type == SPD_MSGTYPE_TEXT ||
+		    message->settings.type == SPD_MSGTYPE_CHAR) {
+			gchar *normalized = g_utf8_normalize(message->buf, -1,
+					G_NORMALIZE_ALL_COMPOSE);
+			if (!normalized) {
+				MSG(2, "Error: Not UTF-8 valid");
+				pthread_mutex_unlock(&element_free_mutex);
+				continue;
+			}
+			if (strcmp(message->buf, normalized)) {
+				MSG(5, "text: Normalized '%s' to '%s'", message->buf, normalized);
+			}
+			message->buf = normalized;
+			insert_symbols(message, punct_missing);
+		}
+
 		/* Insert index marks into textual messages */
 		if (message->settings.type == SPD_MSGTYPE_TEXT) {
 			insert_index_marks(message,
@@ -225,11 +273,12 @@ void *speak(void *data)
 		}
 
 		/* Write the message to the output layer. */
-		ret = output_speak(message);
+		ret = output_speak(message, output);
+
 		MSG(4, "Message sent to output module");
 		if (ret == -1) {
 			MSG(2, "Error: Output module failed");
-			output_check_module(get_output_module(message));
+			output_check_module(output);
 			pthread_mutex_unlock(&element_free_mutex);
 			continue;
 		}
@@ -336,7 +385,7 @@ int reload_message(TSpeechDMessage * msg)
 		im += client_settings->pause_context;
 
 		MSG2(5, "index_marking",
-		     "Requested index mark (with context) is %d (%d+%d)", im,
+		     "Requested index mark (with context) is %d (%s+%d)", im,
 		     msg->settings.index_mark, client_settings->pause_context);
 		if (im < 0) {
 			im = 0;
@@ -441,7 +490,6 @@ void speaking_stop_all()
 	TSpeechDMessage *msg;
 	GList *gl;
 	GList *queue;
-	int gid = -1;
 
 	output_stop();
 
@@ -455,9 +503,7 @@ void speaking_stop_all()
 	assert(gl->data != NULL);
 	msg = (TSpeechDMessage *) gl->data;
 
-	if (msg->settings.reparted != 0) {
-		gid = msg->settings.reparted;
-	} else {
+	if (msg->settings.reparted == 0) {
 		return;
 	}
 
@@ -538,7 +584,7 @@ int speaking_pause(int fd, int uid)
 	settings->paused = 1;
 
 	if (speaking_uid != uid) {
-		MSG(5, "given uid %d not speaking_uid", speaking_uid, uid);
+		MSG(5, "given uid %d not speaking_uid %d", uid, speaking_uid);
 		return 0;
 	}
 
@@ -662,7 +708,6 @@ REPORT_STATE(begin, EVENT_BEGIN_C, EVENT_BEGIN)
 
 int is_sb_speaking(void)
 {
-	int ret;
 	char *index_mark;
 	TFDSetElement *settings;
 
@@ -677,7 +722,7 @@ int is_sb_speaking(void)
 		}
 		settings = &(current_message->settings);
 
-		ret = output_is_speaking(&index_mark);
+		output_is_speaking(&index_mark);
 		if (index_mark == NULL)
 			return SPEAKING = 0;
 
